--[[
Interface: 1.5.1.0 b1580

History:
V 1.0.0.0 @ 08.02.2018

Small script to allow a single extra animation when all default options are in use. Designed to be very light when a single function is needed.


XML Example:
<extraAnimation name="rearDoor" openText="action_openSideDoor" closeText="action_closeSideDoor" setInputButton="sideDoor"/>

modDesc Example:

<specializations>
	<specialization name="extraAnimation" className="ExtraAnimation" filename="scripts/ExtraAnimation.lua" />
</specializations>


---------------------------------------------------------------------------------------------------------------
The following is only required if 'setInputButton' is used. Default key = 'N' (Toggle Cover inputBinding)
---------------------------------------------------------------------------------------------------------------

<l10n>
	<text name="input_sideDoor">
		<en>Open/Close</en>
		<de>Öffnen/schließen</de>
	</text>
</l10n>

<inputBindings>
	<input name="sideDoor" category="VEHICLE" key1="KEY_o" key2="" device="1" button="BUTTON_15" mouse="" visible="true" />
</inputBindings>
]]

ExtraAnimation = {}

function ExtraAnimation.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations)
end

function ExtraAnimation:preLoad(savegame)
	self.extraAnimationState = ExtraAnimation.extraAnimationState
end

function ExtraAnimation:load(savegame)
	
	self.singleAnimation = getXMLString(self.xmlFile, "vehicle.extraAnimation#name")
	if self.singleAnimation ~= nil then
		self.animOpenText = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.extraAnimation#openText"), "action_openDoor")
		self.animCloseText = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.extraAnimation#closeText"), "action_closeDoor")
		
		local setInputButtonStr = getXMLString(self.xmlFile, "vehicle.extraAnimation#setInputButton")
		if setInputButtonStr ~= nil then
			self.setInputButton = InputBinding[setInputButtonStr]
		end

		self.setInputButton = Utils.getNoNil(self.setInputButton, InputBinding.TOGGLE_COVER)
		
		local animIsOpen = false		
		if savegame ~= nil then
            animIsOpen = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#animIsOpen"), animIsOpen)
        end

		self:extraAnimationState(animIsOpen, true)
	else
        self.animIsOpen = false
	end
	
end

function ExtraAnimation:getSaveAttributesAndNodes(nodeIdent)
    local attributes = ""
	local nodes = nil
	
	if self.singleAnimation ~= nil then
        attributes = attributes.. 'animIsOpen="'..tostring(self.animIsOpen)..'"'
    end
	
    return attributes, nodes
end

function ExtraAnimation:readStream(streamId, connection)
    if connection:getIsServer() then		
		if self.singleAnimation ~= nil then
			local animIsOpen = streamReadBool(streamId)
			self:extraAnimationState(animIsOpen, true)
		end
    end
end

function ExtraAnimation:writeStream(streamId, connection)
    if not connection:getIsServer() then		
		if self.singleAnimation ~= nil then
			streamWriteBool(streamId, self.animIsOpen)
		end
    end
end

function ExtraAnimation:update(dt)	
	if self:getIsActiveForInput() then
		if self.singleAnimation ~= nil then
			if InputBinding.hasEvent(self.setInputButton) then
				self:extraAnimationState(not self.animIsOpen)
			end
		end
	end
end

function ExtraAnimation:draw()
	if self.isClient and self:getIsActiveForInput(true) then
		if self.singleAnimation ~= nil then
			if self.animIsOpen then
				g_currentMission:addHelpButtonText(g_i18n:getText(self.animCloseText), self.setInputButton, nil, GS_PRIO_NORMAL)
			else                                                                                                  
				g_currentMission:addHelpButtonText(g_i18n:getText(self.animOpenText), self.setInputButton, nil, GS_PRIO_NORMAL)
			end
        end
    end
end

function ExtraAnimation:extraAnimationState(animIsOpen, noEventSend)
	if self.singleAnimation ~= nil then
        ExtraAnimationStateEvent.sendEvent(self, animIsOpen, noEventSend)
        self.animIsOpen = animIsOpen

        if animIsOpen then
            self:playAnimation(self.singleAnimation, -1, self:getAnimationTime(self.singleAnimation), true)
        else
            self:playAnimation(self.singleAnimation, 1, self:getAnimationTime(self.singleAnimation), true)
        end
    end
end

function ExtraAnimation:mouseEvent(posX, posY, isDown, isUp, button)
end

function ExtraAnimation:keyEvent(unicode, sym, modifier, isDown)
end

function ExtraAnimation:delete()
end

ExtraAnimationStateEvent = {}
ExtraAnimationStateEvent_mt = Class(ExtraAnimationStateEvent, Event)
InitEventClass(ExtraAnimationStateEvent, "ExtraAnimationStateEvent")

function ExtraAnimationStateEvent:emptyNew()
    local self = Event:new(ExtraAnimationStateEvent_mt)
    return self
end

function ExtraAnimationStateEvent:new(object, animOpen)
    local self = ExtraAnimationStateEvent:emptyNew()
    self.object = object
    self.animOpen = animOpen
    return self
end

function ExtraAnimationStateEvent:readStream(streamId, connection)
    self.object = readNetworkNodeObject(streamId)
    self.animOpen = streamReadBool(streamId)
    self:run(connection)
end

function ExtraAnimationStateEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.object)
    streamWriteBool(streamId, self.animOpen)
end

function ExtraAnimationStateEvent:run(connection)
    if not connection:getIsServer() then
        g_server:broadcastEvent(self, false, connection, self.object)
    end
    if self.object ~= nil then
        self.object:extraAnimationState(self.animOpen, true)
    end
end

function ExtraAnimationStateEvent.sendEvent(vehicle, animOpen, noEventSend)
    if vehicle.animIsOpen ~= animOpen then
        if noEventSend == nil or noEventSend == false then
            if g_server ~= nil then
                g_server:broadcastEvent(ExtraAnimationStateEvent:new(vehicle, animOpen), nil, nil, vehicle)
            else
                g_client:getServerConnection():sendEvent(ExtraAnimationStateEvent:new(vehicle, animOpen))
            end
        end
    end
end